"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfig = exports.findCurlyBracedDirectives = exports.chunkFilenames = exports.spawnAsync = exports.wrapOrphanObjectInParens = exports.removeParensWrappingOrphanedObject = exports.dynamicImport = void 0;
const childProcess = require("node:child_process");
const fs = require("node:fs");
const os = require("node:os");
const balanced_match_1 = require("balanced-match");
// Helper function to work around import issues with ESM module
// eslint-disable-next-line no-new-func
exports.dynamicImport = new Function('specifier', 'return import(specifier)');
// From zeke/standard-markdown
function removeParensWrappingOrphanedObject(block) {
    return block.replace(/^\(([{|[][\s\S]+[}|\]])\)$/gm, '$1');
}
exports.removeParensWrappingOrphanedObject = removeParensWrappingOrphanedObject;
// From zeke/standard-markdown
function wrapOrphanObjectInParens(block) {
    return block.replace(/^([{|[][\s\S]+[}|\]])$/gm, '($1)');
}
exports.wrapOrphanObjectInParens = wrapOrphanObjectInParens;
async function spawnAsync(command, args, options) {
    return new Promise((resolve, reject) => {
        try {
            const stdio = { stdout: '', stderr: '' };
            const spawned = childProcess.spawn(command, args, options || {});
            spawned.stdout.on('data', (data) => {
                stdio.stdout += data;
            });
            spawned.stderr.on('data', (data) => {
                stdio.stderr += data;
            });
            spawned.on('exit', (code) => resolve(Object.assign(Object.assign({}, stdio), { status: code })));
            spawned.on('error', (err) => reject(err));
        }
        catch (err) {
            reject(err);
        }
    });
}
exports.spawnAsync = spawnAsync;
function chunkFilenames(filenames, offset = 0) {
    // Windows has a max command line length of 2047 characters, so we can't
    // provide too many filenames without going over that. To work around that,
    // chunk up a list of filenames such that it won't go over that limit when
    // used as args. Use a much higher limit on other platforms which will
    // effectively be a no-op.
    const MAX_FILENAME_ARGS_LENGTH = os.platform() === 'win32' ? 2047 - offset : 100 * 1024;
    return filenames.reduce((chunkedFilenames, filename) => {
        const currChunk = chunkedFilenames[chunkedFilenames.length - 1];
        const currChunkLength = currChunk.reduce((totalLength, _filename) => totalLength + _filename.length + 1, 0);
        if (currChunkLength + filename.length + 1 > MAX_FILENAME_ARGS_LENGTH) {
            chunkedFilenames.push([filename]);
        }
        else {
            currChunk.push(filename);
        }
        return chunkedFilenames;
    }, [[]]);
}
exports.chunkFilenames = chunkFilenames;
function findCurlyBracedDirectives(directive, str) {
    const prefix = `${directive}=`;
    const matches = [];
    let idx = 0;
    while (idx >= 0 && idx < str.length) {
        idx = str.indexOf(prefix, idx);
        if (idx >= 0) {
            idx = idx + prefix.length;
            const val = str.slice(idx);
            const range = (0, balanced_match_1.range)('{', '}', val);
            if (range) {
                matches.push(val.slice(range[0] + 1, range[1]).trim());
            }
        }
    }
    return matches;
}
exports.findCurlyBracedDirectives = findCurlyBracedDirectives;
function loadConfig(path) {
    if (!fs.existsSync(path)) {
        return undefined;
    }
    const config = fs.readFileSync(path, 'utf8');
    try {
        return JSON.parse(config);
    }
    catch (_a) {
        throw new Error(`Couldn't parse config at ${path}`);
    }
}
exports.loadConfig = loadConfig;
//# sourceMappingURL=helpers.js.map